@@ -655,8 +655,8 @@ void ChatMaster::slotConnectApplication(Application *application)
       kDebug() << "Connecting additional signals for InkTransferP2P.";
 #endif
 
-      connect( application, SIGNAL(       inkReceived(const QString&,const QString&) ),
-               this,          SLOT( slotGotInkMessage(const QString&,const QString&) ));
+      connect( application, SIGNAL(       inkReceived(const QString&,const QString&,InkFormat) ),
+               this,          SLOT( slotGotInkMessage(const QString&,const QString&,InkFormat) ));
     }
     else
     {
@@ -1284,24 +1284,17 @@ void ChatMaster::slotDeliverMimeMessage(const MimeMessage &message, const QStrin
 
 
 // Parse an gif ink message
-void ChatMaster::slotGotInkMessage( const QString &inkData, const QString &contactHandle )
+void ChatMaster::slotGotInkMessage( const QString &inkData, const QString &contactHandle, InkFormat format )
 {
 #ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
-  kDebug() << "Parsing ink message";
+  switch( format )
+  {
+    case FORMAT_ISF: kDebug() << "Parsing ISF ink message"; break;
+    case FORMAT_GIF: kDebug() << "Parsing GIF ink message"; break;
+  }
 #endif
 
-  // Validate
-  QRegExp inkChars("([^A-Za-z0-9:+/=])" );
-  if( ! inkData.startsWith("base64:") || inkChars.indexIn( inkData ) != -1 )
-  {
-    kWarning() << "Received invalid ink message from" << contactHandle << ", found char 0x" << QString::number( inkChars.cap(1)[0].unicode(), 16 ) << "at position" << inkChars.pos(1) << "in: " << inkData;
-    return;
-  }
-
-  // Use the ink data as inline image.
-  QString inkUrlData( QString( inkData ).replace( "base64:", "base64," ) );
-
-  // Get the contact data
+  // Get the contact data for the shown messages
   QString friendlyName;
   QString contactPicture;
   const ContactBase *contact = currentAccount_->getContactByHandle( contactHandle );
@@ -1311,17 +1304,108 @@ void ChatMaster::slotGotInkMessage( const QString &inkData, const QString &conta
     contactPicture = contact->getContactPicturePath();
   }
 
-  // Cast the sender object to determinate if the ink is recevied from p2p or switchboard.
+#if KMESS_ENABLE_INK == 0
+  // Receiving Ink is supported only for GIF, as GIF reading is always built-in into Qt
+  // and KHTML can show GIF images without any extra libs
+  if( format != FORMAT_GIF )
+  {
+    showSpecialMessage( ChatMessage( ChatMessage::TYPE_SYSTEM,
+                                     ChatMessage::CONTENT_SYSTEM_ERROR,
+                                     true,
+                                     i18nc( "Error message shown in chat, %1 is the name of the contact",
+                                            "You received an handwritten message from %1, "
+                                            "but it could not be displayed. This version of "
+                                            "KMess was built without ISF support." ),
+                                     contactHandle,
+                                     friendlyName ) );
+    return;
+  }
+#endif
+
+
+  // Validate the incoming data
+  QRegExp inkChars( "([^A-Za-z0-9:+/=])" );
+  if( ! inkData.startsWith( "base64:" ) || inkChars.indexIn( inkData ) != -1 )
+  {
+    kWarning() << "Received invalid ink message from" << contactHandle << ", found char 0x" << QString::number( inkChars.cap(1)[0].unicode(), 16 ) << "at position" << inkChars.pos(1);
+    kWarning() << "Ink data dump:" << inkData;
+
+    showSpecialMessage( ChatMessage( ChatMessage::TYPE_SYSTEM,
+                                     ChatMessage::CONTENT_SYSTEM_ERROR,
+                                     true,
+                                     i18nc( "Error message shown in chat, %1 is the name of the contact",
+                                            "You received an handwritten message from %1, "
+                                            "but it could not be displayed. The data could "
+                                            "not be read." ),
+                                     contactHandle,
+                                     friendlyName ) );
+    return;
+  }
+
+  QString imageMimeType;
+  QByteArray inkEncodedData( inkData.toLatin1() );
+  inkEncodedData.replace( "base64:", "" );
+
+  switch( format )
+  {
+    case FORMAT_ISF:
+    {
+#if KMESS_ENABLE_INK == 1
+      // ISF messages need to be base64-decoded, converted to a picture readable by KHTML,
+      // then base64-encoded again.
+      // Since I'd rather avoid using giflib to encode ISF data into pictures, I'll use PNG.
+      imageMimeType = "image/png";
+
+      QPixmap pixmap;
+
+      Isf::Drawing drawing( Isf::Stream::reader( inkEncodedData, true /* fromBase64 */ ) );
+
+      if( drawing.isNull() )
+      {
+        // Create a blank picture instead of sending invalid data to KHTML
+        pixmap = QPixmap( 10, 10 );
+        pixmap.fill( Qt::white );
+      }
+      else
+      {
+        pixmap = drawing.pixmap();
+      }
+
+      // Save it using a buffer
+      QBuffer buffer( &inkEncodedData );
+      buffer.open( QIODevice::WriteOnly );
+      pixmap.save( &buffer, "PNG" );
+      buffer.close();
+
+      // Encode the data with Base64 to insert it inline in the HTML image tag
+      inkEncodedData = inkEncodedData.toBase64();
+#endif
+      break;
+    }
+
+    case FORMAT_GIF:
+      // Yes, this is it, just set the GIF mimetype. The data is already
+      // there in the inkEncodedData variable, and KHTML can show inline
+      // gif images right away
+      imageMimeType = "image/gif";
+      break;
+  }
+
+  // Use the ink data as inline image
+
+  // Cast the sender object to determine if the ink is received from p2p or switchboard.
   // WLM sends the ink (gif) by p2p protocol for private chat and by mime for multi-chat
-  // Other clients send all by mime message, so it's importat to determinate what switchboard
-  // is used to grep the correct chat window to showing the message
+  // Other clients send all by mime message, so it's important to determine what switchboard
+  // is used, to find the correct chat window where to show the message.
   MsnSwitchboardConnection *switchboard = qobject_cast<MsnSwitchboardConnection *>( sender() );
 
   // Send the message as HTML to the chat window.
   // Using data url scheme for embedded images (src="data:image/gif;base64,fdsfsdfsd")
   showSpecialMessage( ChatMessage( ChatMessage::TYPE_INCOMING,
                                    ChatMessage::CONTENT_MESSAGE_INK,
-                                   true,  "<img src=\"data:image/gif;" + Qt::escape( inkUrlData ) + "\">",
+                                   true,
+                                   "<img src=\"data:" + imageMimeType + ";"
+                                   "base64," + Qt::escape( inkEncodedData ) + "\">",
                                    contactHandle,
                                    friendlyName,
                                    contactPicture ), switchboard );
@@ -1795,8 +1879,8 @@ MsnSwitchboardConnection *ChatMaster::createSwitchboardConnection( MsnSwitchboar
           this,        SLOT  (               slotGotMessage(const MimeMessage&, const QString&) ));
   connect(switchboard, SIGNAL(                   gotMessage(const P2PMessage&,  const QString&) ),
           this,        SLOT  (               slotGotMessage(const P2PMessage&,  const QString&) ));
-  connect(switchboard, SIGNAL(                gotInkMessage(const QString&,     const QString&) ),
-          this,        SLOT  (            slotGotInkMessage(const QString&,     const QString&) ));
+  connect(switchboard, SIGNAL(                gotInkMessage(const QString&,     const QString&,InkFormat) ),
+          this,        SLOT  (            slotGotInkMessage(const QString&,     const QString&,InkFormat) ));
   connect(switchboard, SIGNAL(                 gotMsnObject(const QString&,     const QString&) ),
           this,        SLOT  (             slotGotMsnObject(const QString&,     const QString&) ));
   connect(switchboard, SIGNAL(                    readySend()                                   ),
