@@ -41,8 +41,8 @@
 #include <KMenu>
 
 // The constructor
-ChatMessageView::ChatMessageView( QWidget *parent )
- : KHTMLPart( parent, 0, DefaultGUI )
+ChatMessageView::ChatMessageView( QWidget *parentWidget, QObject *parent )
+ : KHTMLPart( parentWidget, parent, DefaultGUI )
  , chatClearingMark_(0)
  , isEmpty_(true)
  , lastMessageId_(0)
@@ -389,7 +389,7 @@ QString ChatMessageView::getHistory( Account::ChatExportFormat format, bool appe
       // Before the first message, add a simple header
       chatHistory += "\n\n"
                   +  i18nc( "Header of a single chat saved as plain text chat log: %1 is the chat date and time",
-                            "Chat started on: %2",
+                            "Chat started on: %1",
                             KGlobal::locale()->formatDateTime( QDateTime::fromTime_t( date ), KLocale::ShortDate ) )
                   +  "\n------------------------------------------------------------\n";
 
@@ -429,7 +429,7 @@ QString ChatMessageView::getHistory( Account::ChatExportFormat format, bool appe
       if( ! append )
       {
         chatHistory.prepend( i18nc( "Header of a chat file saved in plain text: %1 is the contact",
-                                    "Saved KMess chats with %1" )
+                                    "Saved KMess chats with %1", handle )
                            + "\n######################################################################\n" );
       }
 
