@@ -27,12 +27,15 @@
 #include "../kmess.h"
 #include "../kmessapplication.h"
 #include "contactswidget.h"
-#include "inkedit.h"
+
+#include <IsfQt>
+#include <IsfInkCanvas>
 
 #include <QClipboard>
 #include <QDropEvent>
 #include <QEvent>
 #include <QRegExp>
+#include <QTemporaryFile>
 #include <QTextCodec>
 #include <QTextDocument>
 #include <QTextOption>
@@ -57,7 +60,7 @@ ChatView::ChatView( QWidget *parent )
 , initialized_( false )
 {
   // Insert a KHTMLPart in the placeholder
-  chatMessageView_ = new ChatMessageView( this );
+  chatMessageView_ = new ChatMessageView( this, this );
   connect( chatMessageView_, SIGNAL(                popupMenu(const QString&,const QPoint&)  ),
            this,             SLOT  (      slotShowContextMenu(const QString&,const QPoint&)  ) );
   connect( chatMessageView_, SIGNAL(           openUrlRequest(const KUrl&)                   ),
@@ -68,15 +71,15 @@ ChatView::ChatView( QWidget *parent )
   // Create a layout to maximize the KHTMLPart
   QVBoxLayout *layout = new QVBoxLayout( this );
   layout->setContentsMargins( 0, 6, 0, 0 );
-  layout->addWidget( chatMessageView_->widget() );
+  layout->addWidget( chatMessageView_->view() );
 
   // And to the messages view
   chatMessageView_->view()->setAcceptDrops( true );
   chatMessageView_->view()->installEventFilter( this );
 
   // Create the containers for this chat's current text and drawing
-  inkEditContents_     = new InkImage();
   messageEditContents_ = new QTextDocument();
+  inkDrawing_          = new Isf::Drawing();
 
   // Adapt the new document for RTL is necessary
   if( QApplication::isRightToLeft() )
@@ -95,7 +98,7 @@ ChatView::~ChatView()
 {
   // Delete the related widgets
   delete contactsWidget_;
-  delete inkEditContents_;
+  delete inkDrawing_;
   delete messageEditContents_;
 }
 
@@ -125,20 +128,33 @@ bool ChatView::eventFilter( QObject *obj, QEvent *event )
   switch( event->type() )
   {
     case QEvent::DragEnter:
+    case QEvent::DragMove:
     {
       QDragEnterEvent *dragEvent = static_cast<QDragEnterEvent*>( event );
 
-      if( dragEvent->mimeData()->hasUrls() )
+      if( ! dragEvent )
       {
-        dragEvent->acceptProposedAction();
-        return true;
+#ifdef KMESSDEBUG_CHATVIEW
+          kDebug() << "Ignoring invalid drag event!";
+#endif
+        return false;
       }
-      else if( dragEvent->mimeData()->hasFormat( "application/kmess.list.item" ) )
+
+      const QMimeData *mimeData = dragEvent->mimeData();
+      if( mimeData->hasUrls()
+      ||  mimeData->hasImage()
+      ||  mimeData->hasFormat( "application/kmess.list.item" ) )
       {
+#ifdef KMESSDEBUG_CHATVIEW
+        kDebug() << "Accepting drag event of mimetypes:" << dragEvent->mimeData()->formats();
+#endif
         dragEvent->acceptProposedAction();
         return true;
       }
 
+#ifdef KMESSDEBUG_CHATVIEW
+      kDebug() << "Ignoring invalid drop having mimetypes:" << dragEvent->mimeData()->formats();
+#endif
       break;
     }
 
@@ -147,21 +163,77 @@ bool ChatView::eventFilter( QObject *obj, QEvent *event )
       QDropEvent *dropEvent = static_cast<QDropEvent*>( event );
       const QMimeData *data = dropEvent->mimeData();
 
-      // Process a file transfer drop only if it contains file URLs
-      // and comes from out of our application.
-      if( data->hasUrls() && dropEvent->source() == 0 )
+      // Process the drop only if it comes from out of our application.
+      if( ! dropEvent || dropEvent->source() != 0 )
       {
 #ifdef KMESSDEBUG_CHATVIEW
-        kDebug() << "Drag'n'dropped files:" << dropEvent->mimeData()->urls();
+        kDebug() << "Ignoring invalid drop!";
+#endif
+        break;
+      }
+
+      const QMimeData *mimeData = dropEvent->mimeData();
+      if( mimeData->hasUrls() )
+      {
+#ifdef KMESSDEBUG_CHATVIEW
+        kDebug() << "Drag'n'dropped files:" << mimeData->urls();
 #endif
 
         // Send the files to the contact
-        startFileTransfer( dropEvent->mimeData()->urls() );
+        startFileTransfer( mimeData->urls() );
 
         dropEvent->acceptProposedAction();
         return true;
       }
 
+      if( mimeData->hasImage() )
+      {
+        QImage image( qvariant_cast<QImage>( mimeData->imageData() ) );
+
+        if( image.isNull() )
+        {
+#ifdef KMESSDEBUG_CHATVIEW
+          kDebug() << "Drag'n'dropped an invalid image!";
+#endif
+          return true;
+        }
+
+#ifdef KMESSDEBUG_CHATVIEW
+        kDebug() << "Drag'n'dropped a" << image.size() << "image";
+#endif
+
+        // Create a temporary file where the image will be stored.
+        // TODO: The file is not destroyed, as it needs to be kept
+        // available until the transfer is completed. Find a way
+        // to remove it when KMess quits or as soon as the transfer
+        // is done.
+        QTemporaryFile file;
+        file.setAutoRemove( false );
+        file.setFileTemplate( QDir::tempPath() + "/kmess.XXXXXX.jpg" );
+
+        if( ! file.open() )
+        {
+#ifdef KMESSDEBUG_CHATVIEW
+          kDebug() << "Cannot create temporary image file!";
+#endif
+          return true;
+        }
+
+        dropEvent->acceptProposedAction();
+
+        // Save the image and get its local URL
+        image.save( &file, "JPG" );
+        file.close();
+        QUrl imageUrl( "file://" + file.fileName() );
+
+        // Send the image to the contact
+        QList<QUrl> urlList;
+        urlList.append( imageUrl );
+        startFileTransfer( urlList );
+
+        return true;
+      }
+
       if(   data->hasFormat( "application/kmess.list.item" )
       &&  ! data->text().isEmpty() )
       {
@@ -218,9 +290,9 @@ ContactsWidget *ChatView::getContactsWidget() const
 
 
 // Return a pointer to the ink image drawn by the user in this chat
-InkImage *ChatView::getInkEditContents() const
+Isf::Drawing *ChatView::getInkDrawing() const
 {
-  return inkEditContents_;
+  return inkDrawing_;
 }
 
 
