@@ -34,9 +34,13 @@
 #include "config-kmess.h"
 #include "contactswidget.h"
 #include "emoticonswidget.h"
-#include "inkedit.h"
 #include "winkswidget.h"
 
+#include <IsfQt>
+#include <IsfQtDrawing>
+
+#include <QBuffer>
+#include <QDir>
 #include <QDockWidget>
 #include <QShortcut>
 #include <QTextDocumentFragment>
@@ -86,9 +90,9 @@ ChatWindow::ChatWindow( QWidget *parent )
   setCentralWidget( mainWidget );
 
 #if KMESS_ENABLE_INK == 0
-  // Only enable the Ink button if GIFLib is available
+  // Only enable the Ink button if Ink sending support is available
   inkButton_->hide();
-  inkEdit_->setEnabled( false );
+  inkCanvas_->setEnabled( false );
 #endif
 
   // Connect the UI signals
@@ -98,7 +102,7 @@ ChatWindow::ChatWindow( QWidget *parent )
            this,                     SLOT  ( slotNewLineButtonClicked() ) );
   connect( messageEdit_,             SIGNAL(              textChanged() ),
            this,                     SLOT  (       slotMessageChanged() ) );
-  connect( inkEdit_,                 SIGNAL(               inkChanged() ),
+  connect( inkCanvas_,               SIGNAL(               inkChanged() ),
            this,                     SLOT  (       slotMessageChanged() ) );
   connect( textButton_,              SIGNAL(                  clicked() ),
            this,                     SLOT  (         slotSwitchEditor() ) );
@@ -110,6 +114,10 @@ ChatWindow::ChatWindow( QWidget *parent )
            this,                     SLOT  (         slotSwitchEditor() ) );
   connect( customEmoticonButton_,    SIGNAL(                  clicked() ),
            this,                     SLOT  (         slotSwitchEditor() ) );
+  connect( fontButton_,              SIGNAL(                  clicked() ),
+           this,                     SLOT  (                 editFont() ) );
+  connect( fontColorButton_,         SIGNAL(                  clicked() ),
+           this,                     SLOT  (            editFontColor() ) );
 
   // Create the contacts dock
   contactsDock_ = new QDockWidget( i18n( "Contacts" ), this );
@@ -142,7 +150,7 @@ ChatWindow::ChatWindow( QWidget *parent )
 
   //Install the event filters to catch events
   installEventFilter( this );
-  inkEdit_    ->installEventFilter( this );
+  inkCanvas_  ->installEventFilter( this );
   messageEdit_->installEventFilter( this );
   winksWidget_->installEventFilter( this );
   messageEdit_->viewport()->installEventFilter( this );
@@ -153,6 +161,24 @@ ChatWindow::ChatWindow( QWidget *parent )
   inkButton_             ->setIcon( KIcon( "draw-brush" ) );
   winksButton_           ->setIcon( KIcon( "applications-toys" ) );
   textButton_            ->setIcon( KIcon( "draw-text" ) );
+  fontButton_            ->setIcon( KIcon( "preferences-desktop-font" ) );
+  fontColorButton_       ->setIcon( KIcon( "format-stroke-color" ) );
+
+  // Set up the buttons for setting ink colour/stroke size/etc.
+  inkCanvas_->setPenSize( inkPenSize_->value() );
+
+  inkColorButton_->setIcon( KIcon( "format-stroke-color" ) );
+  inkEraseButton_->setIcon( KIcon( "draw-eraser"         ) );
+  inkClearButton_->setIcon( KIcon( "edit-clear"          ) );
+
+  connect( inkColorButton_,        SIGNAL(            clicked()    ),
+           this,                   SLOT  ( slotChangeInkColor()    ) );
+  connect( inkEraseButton_,        SIGNAL(            clicked()    ),
+           this,                   SLOT  ( slotChangeInkBrush()    ) );
+  connect( inkClearButton_,        SIGNAL(            clicked()    ),
+           inkCanvas_,             SLOT  (              clear()    ) );
+  connect( inkPenSize_,            SIGNAL(       valueChanged(int) ),
+           inkCanvas_,             SLOT  (         setPenSize(int) ) );
 
   // Connect the message edit so that if its displayed color changes,
   // it's checked to match the user's chosen color.
@@ -205,6 +231,8 @@ ChatWindow::ChatWindow( QWidget *parent )
   statusTimer_.setSingleShot( true );
   connect( &statusTimer_,  SIGNAL(           timeout() ),
            this,           SLOT  ( showStatusMessage() ) );
+
+  inkCtlFrame_->setVisible( false );
 }
 
 
@@ -333,15 +361,17 @@ bool ChatWindow::checkAndCloseWindow()
 // The chat window is closing
 bool ChatWindow::closeAllTabs()
 {
-  // Do not ask if there only is one tab, or if the user has
-  // explicitly requested all the tabs to be closed
-  if( chatTabs_->count() > 1 && sender() != closeAllAction_ )
+  // Ask the user for confirmation
+  if( chatTabs_->count() > 1       // Do not ask if there only is one tab
+  &&  sender() != closeAllAction_  // Do not ask if the user chose the "close all tabs" action
+  &&  ! kapp->sessionSaving() )    // Do not ask if the KDE session is closing
   {
     int res = KMessageBox::questionYesNoCancel( this,
-                                                i18n( "<html>There are multiple tabs open in this chat window. Do you want to close the "
-                                                "current tab only, or all tabs?<br /><br />"
-                                                "<i>Note: You can close all tabs at once by pressing Alt+F4.</i></html>" ),
-                                                i18nc( "Dialog box caption: closing a chatwindow with a single tab", "Closing a Chat Tab" ),
+                                                i18n( "<html>There are multiple tabs open in this chat window. "
+                                                      "Do you want to close the current tab only, or all tabs?<br /><br />"
+                                                      "<i>Note: You can close all tabs at once by pressing Alt+F4.</i></html>" ),
+                                                i18nc( "Dialog box caption: closing a chatwindow with a single tab",
+                                                       "Closing a Chat Tab" ),
                                                 KGuiItem( i18n( "Close All Tabs" ), "window-close" ), // Yes option
                                                 KGuiItem( i18n( "Close Current Tab" ), "tab-close" ), // No option
                                                 KStandardGuiItem::cancel(),
@@ -461,7 +491,7 @@ void ChatWindow::createMenus()
 
   // Enable some default action shortcuts (preserve the original ones where needed)
   nudgeAction_->setShortcut( QKeySequence( "Alt+Z" ) );
-  closeAction->setShortcuts( QList<QKeySequence>() << QKeySequence( "Ctrl+W" ) << QKeySequence( "Esc" ) << closeAction->shortcut()  );
+  closeAction->setShortcuts( QList<QKeySequence>() << QKeySequence( "Ctrl+W" ) << closeAction->shortcut()  );
   closeAllAction_->setShortcuts( QList<QKeySequence>() << QKeySequence( "Ctrl+Q" ) << closeAllAction_->shortcut()  );
   prevTabAction_->setShortcuts( QList<QKeySequence>() << QKeySequence( QKeySequence::PreviousChild ) << prevTabAction_->shortcut() );
   nextTabAction_->setShortcuts( QList<QKeySequence>() << QKeySequence( QKeySequence::NextChild ) << nextTabAction_->shortcut() );
@@ -505,7 +535,7 @@ void ChatWindow::createMenus()
 
   // Connect slots to signals for "Chat" menu
   connect( inviteButton_,       SIGNAL(           triggered(bool) ),
-           this,                SLOT  (       slotInviteContact() ) );
+           this,                SLOT  (   slotInviteContact()     ) );
   connect( sendAction_,         SIGNAL(           triggered(bool) ),
            this,                SLOT  (   startFileTransfer()     ) );
   connect( nudgeAction_,        SIGNAL(           triggered(bool) ),
@@ -660,6 +690,10 @@ void ChatWindow::editorColorChanged(const QColor &color)
     messageEdit_->setTextColor( QColor( currentAccount_->getFontColor() ) );
     messageEdit_->setFont( QFont ( currentAccount_->getFont() ) );
   }
+
+  // Adjust the message editor's minimum height to match the current font size
+  QFontMetrics metrics( messageEdit_->currentFont() );
+  messageEdit_->setMinimumSize( 100, metrics.height() + metrics.lineSpacing() );
 }
 
 
@@ -692,18 +726,29 @@ bool ChatWindow::eventFilter( QObject *obj, QEvent *event )
     {
       messageEdit_->setFocus();
     }
-    else if( inkEdit_->isVisible() )
+    else if( inkCanvas_->isVisible() )
     {
-      inkEdit_->inkFrame_->setFocus();
+      inkCanvas_->setFocus();
     }
 
     return false;
   }
-
+  
+  if ( event->type() == QEvent::KeyPress )
+  {
+    QKeyEvent *keyEvent = static_cast<QKeyEvent*>( event );
+    if ( keyEvent->key() == Qt::Key_Escape )
+    {
+      closeWidgetOrTab();
+      return true;
+    }
+  }
+  
   // Only manage events for the editors from now on (the event filter code
   // is complex enough as it is now...)
-  if( obj != messageEdit_ && obj != messageEdit_->viewport()
-  &&  obj != inkEdit_
+  if( obj != messageEdit_
+  &&  obj != messageEdit_->viewport()
+  &&  obj != inkCanvas_
   &&  obj != winksWidget_
   &&  obj != standardEmoticonsWidget_
   &&  obj != customEmoticonsWidget_ )
@@ -738,10 +783,11 @@ bool ChatWindow::eventFilter( QObject *obj, QEvent *event )
         keyEvent->ignore();
         return true;
       }
-      // Detect scrolling commands. Shift scrolls faster
-      else if( key == Qt::Key_PageUp || key == Qt::Key_PageDown )
+      // Detect scrolling commands. Alt scrolls faster (shift is used for text selection in the editor)
+      else if( ( key == Qt::Key_PageUp || key == Qt::Key_PageDown ) && ( ! shift ) )
       {
-        getCurrentChat()->scrollTo( ( key == Qt::Key_PageDown ), shift );
+        getCurrentChat()->scrollTo( ( key == Qt::Key_PageDown ), alt );
+        return true;
       }
       else if( alt )
       {
@@ -848,67 +894,135 @@ bool ChatWindow::eventFilter( QObject *obj, QEvent *event )
       {
         messageEdit_->setFocus();
       }
-      else if( inkEdit_->isVisible() )
+      else if( inkCanvas_->isVisible() )
       {
-        inkEdit_->inkFrame_->setFocus();
+        inkCanvas_->setFocus();
       }
       break;
 
     case QEvent::FocusOut:
     case QEvent::Show:
+    {
+      QWidget *focusedWidget = QApplication::focusWidget();
+
+      // Allow this chat's search bar to take focus away from the message editor
+      if( focusedWidget && QString( focusedWidget->metaObject()->className() ) == "KHistoryComboBox" )
       {
-        QWidget *focusedWidget = QApplication::focusWidget();
-
-        // Allow this chat's search bar to take focus away from the message editor
-        if( focusedWidget && QString( focusedWidget->metaObject()->className() ) == "KHistoryComboBox" )
-        {
-          break;
-        }
-
-        messageEdit_->setFocus( Qt::OtherFocusReason );
+        break;
       }
+
+      messageEdit_->setFocus( Qt::OtherFocusReason );
       break;
+    }
 
     case QEvent::DragEnter:
+    case QEvent::DragMove:
+    {
+      QDragEnterEvent *dragEvent = static_cast<QDragEnterEvent*>( event );
+
+      if( ! dragEvent )
       {
-        QDragEnterEvent *dragEvent = static_cast<QDragEnterEvent*>( event );
-
-        if( ! dragEvent || ! dragEvent->mimeData()->hasUrls() )
-        {
 #ifdef KMESSDEBUG_CHATWINDOW
-          kDebug() << "Ignoring invalid drop";
+        kDebug() << "Ignoring invalid drag event!";
 #endif
-          break;
-        }
-
-        dragEvent->acceptProposedAction();
+        return false;
       }
+
+      const QMimeData *mimeData = dragEvent->mimeData();
+      if( mimeData->hasUrls()
+      ||  mimeData->hasImage() )
+      {
+#ifdef KMESSDEBUG_CHATWINDOW
+        kDebug() << "Accepting drag event of mimetypes:" << dragEvent->mimeData()->formats();
+#endif
+        dragEvent->acceptProposedAction();
+        return true;
+      }
+
+#ifdef KMESSDEBUG_CHATWINDOW
+      kDebug() << "Ignoring invalid drop having mimetypes:" << dragEvent->mimeData()->formats();
+#endif
+
       break;
+    }
 
     case QEvent::Drop:
+    {
+      QDropEvent *dropEvent = static_cast<QDropEvent*>( event );
+
+      // Process the drop only if it comes from out of our application.
+      if( ! dropEvent || dropEvent->source() != 0 )
       {
-        QDropEvent *dropEvent = static_cast<QDropEvent*>( event );
-
-        // Process the drop only if it contains file URLs and comes from out of our application.
-        if( ! dropEvent || ! dropEvent->mimeData()->hasUrls() || dropEvent->source() != 0 )
-        {
 #ifdef KMESSDEBUG_CHATWINDOW
-          kDebug() << "Ignoring invalid drop";
+        kDebug() << "Ignoring invalid drop!";
 #endif
-          break;
-        }
+        break;
+      }
 
+      const QMimeData *mimeData = dropEvent->mimeData();
+      if( mimeData->hasUrls() )
+      {
 #ifdef KMESSDEBUG_CHATWINDOW
-        kDebug() << "Drag'n'dropped files:" << dropEvent->mimeData()->urls();
+        kDebug() << "Drag'n'dropped files:" << mimeData->urls();
 #endif
 
         // Send the files to the contact
-        getCurrentChat()->startFileTransfer( dropEvent->mimeData()->urls() );
+        getCurrentChat()->startFileTransfer( mimeData->urls() );
 
         dropEvent->acceptProposedAction();
         return true;
       }
+
+      if( mimeData->hasImage() )
+      {
+        QImage image( qvariant_cast<QImage>( mimeData->imageData() ) );
+
+        if( image.isNull() )
+        {
+#ifdef KMESSDEBUG_CHATVIEW
+          kDebug() << "Drag'n'dropped an invalid image!";
+#endif
+          return true;
+        }
+
+#ifdef KMESSDEBUG_CHATWINDOW
+        kDebug() << "Drag'n'dropped a" << image.size() << "image";
+#endif
+
+        // Create a temporary file where the image will be stored.
+        // TODO: The file is not destroyed, as it needs to be kept
+        // available until the transfer is completed. Find a way
+        // to remove it when KMess quits or as soon as the transfer
+        // is done.
+        QTemporaryFile file;
+        file.setAutoRemove( false );
+        file.setFileTemplate( QDir::tempPath() + "/kmess.XXXXXX.jpg" );
+
+        if( ! file.open() )
+        {
+#ifdef KMESSDEBUG_CHATWINDOW
+          kDebug() << "Cannot create temporary image file!";
+#endif
+          return true;
+        }
+
+        dropEvent->acceptProposedAction();
+
+        // Save the image and get its local URL
+        image.save( &file, "JPG" );
+        file.close();
+        QUrl imageUrl( "file://" + file.fileName() );
+
+        // Send the image to the contact
+        QList<QUrl> urlList;
+        urlList.append( imageUrl );
+        getCurrentChat()->startFileTransfer( urlList );
+
+        return true;
+      }
+
       break;
+    }
 
     default:
       break;
@@ -1325,17 +1439,91 @@ void ChatWindow::saveProperties( KConfigGroup &config )
 // Send a ink via server
 void ChatWindow::sendInk()
 {
-  // Get the image bytes and send them
-  const QByteArray& ink( inkEdit_->getImageBytes() );
+  bool supportGifInk = true;
+  bool supportIsfInk = true;
 
-  getCurrentChat()->sendInkMessage( ink );
+  // Find out which ink formats are supported by all participants
+  const QStringList &participants( getCurrentChat()->getParticipants() );
+  foreach( const QString &handle, participants )
+  {
+    const ContactBase *contact = currentAccount_->getContactByHandle( handle );
+    if( contact == 0 )
+    {
+      continue;
+    }
 
-  // Clear and reset the state of send button
-  inkEdit_->clearImage();
-  sendButton_->setEnabled( false );
+    supportGifInk &= contact->hasCapability( ContactBase::MSN_CAP_INK_GIF );
+    supportIsfInk &= contact->hasCapability( ContactBase::MSN_CAP_INK_ISF );
+  }
+
+  // The contacts don't support either formats! Something is wrong, since the ink editor
+  // was enabled...
+  if( ! supportGifInk && ! supportIsfInk )
+  {
+    kWarning() << "Neither Ink formats are supported by the participants!";
+
+    getCurrentChat()->showMessage( ChatMessage( ChatMessage::TYPE_SYSTEM,
+                                                ChatMessage::CONTENT_SYSTEM_ERROR,
+                                                true,
+                                                i18nc( "Error message shown in chat",
+                                                       "Failed to send the handwritten message: "
+                                                       "the contacts do not support it." ),
+                                                QString(),
+                                                QString() ) );
+    return;
+  }
+
+  InkFormat format;
+  QByteArray ink;
+
+  if( supportIsfInk )
+  {
+#ifdef KMESSDEBUG_CHATWINDOW
+    kDebug() << "Retrieving ISF ink data";
+#endif
+
+    format = FORMAT_ISF;
+    ink = Isf::Stream::writer( *( inkCanvas_->drawing() ) );
+  }
+  else if( supportGifInk )
+  {
+#ifdef KMESSDEBUG_CHATWINDOW
+    kDebug() << "Retrieving GIF ink data";
+#endif
+
+    format = FORMAT_GIF;
+    ink = Isf::Stream::writerGif( *( inkCanvas_->drawing() ) );
+  }
+  // Any other possibility has been excluded above
+
+  if( ink.isEmpty() )
+  {
+    kWarning() << "An error occured while creating the" << ( supportIsfInk ? "ISF" : "GIF" ) << "Ink data!";
+
+    getCurrentChat()->showMessage( ChatMessage( ChatMessage::TYPE_SYSTEM,
+                                                ChatMessage::CONTENT_SYSTEM_ERROR,
+                                                true,
+                                                i18nc( "Error message shown in chat",
+                                                       "Failed to send the handwritten message: "
+                                                       "an error has occurred while creating it." ),
+                                                QString(),
+                                                QString() ) );
+    return;
+  }
+
+  getCurrentChat()->sendInkMessage( format, ink );
+
+  // Get the pixels, to show the ink in chat
+  QImage image( inkCanvas_->image() );
+
+  // Save it as a PNG image, which KHTML will be able to show
+  QBuffer buffer;
+  buffer.open( QIODevice::WriteOnly );
+  image.save( &buffer, "PNG" );
+  buffer.close();
 
   // Show the message in the browser window
-  const QString& text( "<img src='data:image/gif;base64," + ink.toBase64() + "'>" );
+  const QString& text( "<img src='data:image/png;base64," + buffer.data().toBase64() + "'>" );
 
   ChatMessage message( ChatMessage::TYPE_OUTGOING,
                        ChatMessage::CONTENT_MESSAGE_INK,
@@ -1343,9 +1531,13 @@ void ChatWindow::sendInk()
                        text,
                        currentAccount_->getHandle(),
                        currentAccount_->getFriendlyName( STRING_ORIGINAL ),
-                       currentAccount_->getPicturePath());
+                       currentAccount_->getPicturePath() );
 
-  getCurrentChat()->showMessage(message);
+  getCurrentChat()->showMessage( message );
+
+  // Clear and reset the state of the send button
+  inkCanvas_->clear();
+  sendButton_->setEnabled( false );
 }
 
 
@@ -1370,18 +1562,21 @@ void ChatWindow::sendMessage()
   // Messages like "//hello" should be sent as "/hello"
   // Messages like "/hello" are commands.
   // Check if this is a command, and if so, emit handleCommand signal
-  if( text.left( 2 ) == "//" )
+  if( text.size() > 2 && text.mid( 2, 1 ) != "/" )
   {
-    text = text.mid( 1 );
-  }
-  else if( text.left( 1 ) == "/" )
-  {
-    const QString& command( text.mid( 1 ) );
-
-    // Handle command. If 'false' is returned, send it anyway.
-    if ( handleCommand( command ) )
+    if( text.left( 2 ) == "//" )
     {
-      return;
+      text = text.mid( 1 );
+    }
+    else if( text.left( 1 ) == "/" )
+    {
+      const QString& command( text.mid( 1 ) );
+
+      // Handle command. If 'false' is returned, send it anyway.
+      if ( handleCommand( command ) )
+      {
+        return;
+      }
     }
   }
 
@@ -1503,12 +1698,14 @@ void ChatWindow::setEnabled( bool isEnabled )
   customEmoticonsDock_    ->setEnabled( isEnabled );
   standardEmoticonsDock_  ->setEnabled( isEnabled );
 
-  inkEdit_                ->setEnabled( isEnabled );
+  inkCanvas_              ->setEnabled( isEnabled );
   messageEdit_            ->setEnabled( isEnabled );
   newLineButton_          ->setEnabled( isEnabled );
   sendButton_             ->setEnabled( isEnabled );
   standardEmoticonButton_ ->setEnabled( isEnabled );
   customEmoticonButton_   ->setEnabled( isEnabled );
+  fontButton_             ->setEnabled( isEnabled );
+  fontColorButton_        ->setEnabled( isEnabled );
   inviteButton_           ->setEnabled( isEnabled );
   textButton_             ->setEnabled( isEnabled );
 
@@ -1691,6 +1888,38 @@ void ChatWindow::showStatusMessage( QString message, int duration )
 
 
 
+
+// Change color for current pen
+void ChatWindow::slotChangeInkColor()
+{
+  QColor color;
+  if ( KColorDialog::getColor( color ) == QDialog::Accepted )
+  {
+    inkCanvas_->setPenColor( color );
+  }
+}
+
+
+
+// Erase brush was selected
+void ChatWindow::slotChangeInkBrush()
+{
+  if( inkCanvas_->penType() == Isf::InkCanvas::DrawingPen )
+  {
+    inkCanvas_->setPenType( Isf::InkCanvas::EraserPen );
+    inkEraseButton_->setIcon( KIcon( "draw-freehand" ) );
+    inkEraseButton_->setToolTip( i18n( "Drawing brush" ) );
+  }
+  else
+  {
+    inkCanvas_->setPenType( Isf::InkCanvas::DrawingPen );
+    inkEraseButton_->setIcon( KIcon( "draw-eraser" ) );
+    inkEraseButton_->setToolTip( i18n( "Erase brush" ) );
+  }
+}
+
+
+
 // Make the caption blink if the window still doesn't have focus
 void ChatWindow::slotBlinkCaption()
 {
@@ -1937,7 +2166,7 @@ void ChatWindow::slotMessageChanged()
   // The ink drawing was changed
   else
   {
-    disable = ( inkEdit_->isEmpty() );
+    disable = ( inkCanvas_->isEmpty() );
   }
 
   // Something has been typed, enable the button so the user can send the message
@@ -1994,7 +2223,7 @@ void ChatWindow::slotSendButtonClicked()
   if( inkButton_->isChecked() )
   {
     sendInk();
-    inkEdit_->inkFrame_->setFocus();
+    inkCanvas_->setFocus();
   }
   else if( winksButton_->isChecked() )
   {
@@ -2024,6 +2253,15 @@ void ChatWindow::slotSwitchEditor()
   kDebug() << "Switching editors: caller:" << caller;
 #endif
 
+  // Some widgets are pretty high, hide them unless they're needed
+  standardEmoticonsWidget_->hide();
+  customEmoticonsWidget_->hide();
+  winksWidget_->hide();
+
+  // Show the text control frame by default
+  inkCtlFrame_->setVisible( false );
+  textCtlFrame_->setVisible( true );
+
   // Called by the winks button, show the available winks
   if( caller == winksButton_ && winksButton_->isChecked() )
   {
@@ -2031,7 +2269,10 @@ void ChatWindow::slotSwitchEditor()
     kDebug() << "Switching to winks list.";
 #endif
     winksWidget_->refresh();
+    winksWidget_->show();
+
     editorChooser_->setCurrentWidget( winksPage_ );
+
     winksButton_->setChecked( true );
     inkButton_->setChecked( false );
     standardEmoticonButton_->setChecked( false );
@@ -2053,7 +2294,10 @@ void ChatWindow::slotSwitchEditor()
     customEmoticonButton_->setChecked( false );
     textButton_->setChecked( false );
 
-    disableSendButton = ( inkEdit_->isEmpty() );
+    inkCtlFrame_->setVisible( true );
+    textCtlFrame_->setVisible( false );
+
+    disableSendButton = ( inkCanvas_->isEmpty() );
   }
   // Called by the standard emoticon button
   else if( caller == standardEmoticonButton_
@@ -2062,7 +2306,10 @@ void ChatWindow::slotSwitchEditor()
 #ifdef KMESSDEBUG_CHATWINDOW
     kDebug() << "Switching to standard emoticons widget.";
 #endif
+    standardEmoticonsWidget_->show();
+
     editorChooser_->setCurrentWidget( stdEmoPage_ );
+
     winksButton_->setChecked( false );
     inkButton_->setChecked( false );
     standardEmoticonButton_->setChecked( true );
@@ -2076,7 +2323,10 @@ void ChatWindow::slotSwitchEditor()
 #ifdef KMESSDEBUG_CHATWINDOW
     kDebug() << "Switching to custom emoticons widget.";
 #endif
+    customEmoticonsWidget_->show();
+
     editorChooser_->setCurrentWidget( cusEmoPage_ );
+
     winksButton_->setChecked( false );
     inkButton_->setChecked( false );
     standardEmoticonButton_->setChecked( false );
@@ -2187,7 +2437,6 @@ void ChatWindow::slotTabChanged( int currentIndex )
     messageEdit_->setDocument( new QTextDocument( messageEdit_ ) );
     updateModeButtons();
     updateEditorFont();
-    inkEdit_->setImage( 0 );
     return;
   }
 
@@ -2217,12 +2466,12 @@ void ChatWindow::slotTabChanged( int currentIndex )
   // Display the new chat contents in the UI. Block the signals to avoid the editors
   // from firing the "message changed" signal, which sends a typing message.
   messageEdit_ ->blockSignals( true );
-  inkEdit_     ->blockSignals( true );
+  inkCanvas_   ->blockSignals( true );
   contactsDock_->setWidget( chat->getContactsWidget() );
   messageEdit_ ->setDocument( chat->getMessageEditContents() );
-  inkEdit_     ->setImage( chat->getInkEditContents() );
+  inkCanvas_   ->setDrawing( chat->getInkDrawing() );
   messageEdit_ ->blockSignals( false );
-  inkEdit_     ->blockSignals( false );
+  inkCanvas_   ->blockSignals( false );
 
   // Reset quick re-type index
   indexSentences_ = chat->getQuickRetypeList().size();
@@ -2435,14 +2684,13 @@ void ChatWindow::updateModeButtons()
     return;
   }
 
-#if KMESS_ENABLE_INK == 1
   bool enableInkButton   = true;
-#else
-  bool enableInkButton   = false;
-#endif
-
   bool enableWinksButton = true;
 
+#if KMESS_ENABLE_INK == 0
+  enableInkButton = false;
+#endif
+
   const QStringList &participants( getCurrentChat()->getParticipants() );
   foreach( const QString &handle, participants )
   {
@@ -2455,17 +2703,9 @@ void ChatWindow::updateModeButtons()
 #if KMESS_ENABLE_INK == 1
     if( enableInkButton )
     {
-      // Disable the Ink button if the contact doesn't support GIF Ink
-      if( ! contact->hasCapability( ContactBase::MSN_CAP_INK_GIF ) )
-      {
-        enableInkButton = false;
-      }
-
-      // TODO Fix sending Ink to WLM 8.5 and 2009 (probably by implementing ISF support)
-      // Disable the Ink button also if we're chatting with an ISF-enabled WLM8.5 / WLM2009
-      if( ( contact->getClientName() == "Windows Live Messenger 8.5"
-         || contact->getClientName() == "Windows Live Messenger 2009" )
-      &&  contact->hasCapability( ContactBase::MSN_CAP_INK_ISF ) )
+      // Disable the Ink button if the contact doesn't support Ink
+      if( ! contact->hasCapability( ContactBase::MSN_CAP_INK_GIF )
+      &&  ! contact->hasCapability( ContactBase::MSN_CAP_INK_ISF ) )
       {
         enableInkButton = false;
       }
@@ -2491,7 +2731,7 @@ void ChatWindow::updateModeButtons()
   // Set a tooltip or a status bar message to either buttons.
   // The tooltip covers the status message otherwise.
 
-#if KMESS_ENABLE_INK == 1 // else, the ink button isn't showed anyway
+#if KMESS_ENABLE_INK == 1 // else, the ink button isn't shown anyway
   if( enableInkButton )
   {
     inkButton_->setToolTip  ( i18n( "Handwriting mode" ) );
@@ -2503,14 +2743,14 @@ void ChatWindow::updateModeButtons()
     if( getCurrentChat()->getParticipants().count() > 1 )
     {
       inkButton_->setStatusTip( i18nc( "Label text",
-                                       "Handwriting is disabled: KMess cannot send drawings to "
-                                       "some of the contacts." ) );
+                                       "Handwriting is disabled: some of the contacts do not "
+                                       "support receiving handwritten messages." ) );
     }
     else
     {
       inkButton_->setStatusTip( i18nc( "Label text",
-                                       "Handwriting is disabled: KMess cannot send drawings to "
-                                       "this contact." ) );
+                                       "Handwriting is disabled: this contact does not "
+                                       "support receiving handwritten messages." ) );
     }
   }
 #endif
@@ -2526,14 +2766,14 @@ void ChatWindow::updateModeButtons()
     if( getCurrentChat()->getParticipants().count() > 1 )
     {
       winksButton_->setStatusTip( i18nc( "Label text",
-                                       "Winks are disabled: KMess cannot send winks to "
-                                       "some of the contacts." ) );
+                                       "Winks are disabled: some of the contacts do not "
+                                       "support receiving winks." ) );
     }
     else
     {
       winksButton_->setStatusTip( i18nc( "Label text",
-                                       "Winks are disabled: KMess cannot send winks to "
-                                       "this contact." ) );
+                                       "Winks are disabled: this contact does not "
+                                       "support receiving winks." ) );
     }
   }
 }
