@@ -759,8 +759,8 @@ void KMess::checkAutologin(QString handle)
     // Set the login info in the initial view and connect
     // Using the reconnect() method here instead of startConnecting()
     // so KMess waits until the network is back up.
-    // If the network status is unknown, then connect immediately.
-    initialView_->reconnect( loginAccount->getHandle(), false );
+    // Also connect immediately if possible.
+    initialView_->reconnect( loginAccount->getHandle(), true );
 
     // FIXME for 2.1: when using --autologin, don't display ".. to reconnect" but "to connect".
   }
@@ -864,7 +864,7 @@ void KMess::connectWithAccount(Account *account)
 
 
 // Connect to the server with the given account, possibly temporary or new.
-void KMess::connectWithAccount(QString handle, QString password, bool rememberAccount, bool rememberPassword, Status initialStatus )
+void KMess::connectWithAccount(QString handle, QString password, bool rememberAccount, bool rememberPassword, bool autologin, Status initialStatus )
 {
 #ifdef KMESSDEBUG_KMESS
   kDebug() << "Connecting with handle " << handle;
@@ -880,6 +880,23 @@ void KMess::connectWithAccount(QString handle, QString password, bool rememberAc
     account->setTemporaryPassword( password );
     account->setSavePassword( rememberPassword );
     account->setInitialStatus( initialStatus );
+
+    if( rememberAccount && rememberPassword && autologin )
+    {
+      // Mark this account as the auto-login one
+      const QList<Account*> accountsList = AccountsManager::instance()->getAccounts();
+      foreach( Account *accountItem, accountsList )
+      {
+        // We are searching for a saved account with the autologin flag on
+        if( accountItem->getUseAutologin() )
+        {
+          accountItem->setUseAutologin( false );
+        }
+      }
+
+      account->setUseAutologin( true );
+    }
+
     startConnection( account );
     return;
   }
@@ -893,6 +910,22 @@ void KMess::connectWithAccount(QString handle, QString password, bool rememberAc
   account->setInitialStatus( initialStatus );
   AccountsManager::instance()->addAccount( account );
 
+  if( rememberAccount && rememberPassword && autologin )
+  {
+    // Mark this account as the auto-login one
+    const QList<Account*> accountsList = AccountsManager::instance()->getAccounts();
+    foreach( Account *accountItem, accountsList )
+    {
+      // We are searching for a saved account with the autologin flag on
+      if( accountItem->getUseAutologin() )
+      {
+        accountItem->setUseAutologin( false );
+      }
+    }
+
+    account->setUseAutologin( true );
+  }
+
   // Connect it
   startConnection( account );
 }
@@ -1870,6 +1903,10 @@ void KMess::switchViewToContactList()
   connect( view_,                      SIGNAL( unblockContact(QString)                  ),   // Unblock the contact
            msnNotificationConnection_, SLOT  ( unblockContact(QString)                  ) );
 
+  // this is horrible. i'm glad CurrentAccount is gone in 2.1!
+  Account *realAccount = AccountsManager::instance()->getAccountByHandle( currentAccount_->getHandle() );
+  connect( realAccount, SIGNAL( changedEmoticonSettings() ), view_, SLOT( slotUpdateUserStatus() ) );
+
   // Set it as the application's main widget
   setCentralWidget( view_ );
 }
@@ -1897,8 +1934,8 @@ void KMess::switchViewToInitialScreen()
   initialView_ = new InitialView( this );
 
   // Connect it so it triggers the login process
-  connect( initialView_, SIGNAL( connectWithAccount(QString,QString,bool,bool,Status) ),
-           this,         SLOT  ( connectWithAccount(QString,QString,bool,bool,Status) ) );
+  connect( initialView_, SIGNAL( connectWithAccount(QString,QString,bool,bool,bool,Status) ),
+           this,         SLOT  ( connectWithAccount(QString,QString,bool,bool,bool,Status) ) );
   connect( initialView_, SIGNAL(                                  disconnectClicked() ),
            this,         SLOT  (                                  disconnectClicked() ) );
 
