@@ -121,6 +121,11 @@ KMessView::~KMessView()
   delete contactCopyMapper_;
   delete contactMoveMapper_;
 
+  delete selectionModel_;
+  delete viewModel_;
+  selectionModel_ = 0;
+  viewModel_ = 0;
+  
 #ifdef KMESSDEBUG_KMESSVIEW_GENERAL
   kDebug() << "DESTROYED.";
 #endif
@@ -384,11 +389,6 @@ void KMessView::slotDisconnected()
   updateTimer_.stop();
 
   contactListView_->setModel( 0 );
-
-  delete selectionModel_;
-  delete viewModel_;
-  selectionModel_ = 0;
-  viewModel_ = 0;
 }
 
 
@@ -699,7 +699,7 @@ bool KMessView::initGroupPopup()
   moveGroupDown_ = new KAction( KIcon("arrow-down"),  i18n("Move Group &Down"), this );
   moveGroupUp_   = new KAction( KIcon("arrow-up"),    i18n("Move Group &Up"),   this );
   removeGroup_   = new KAction( KIcon("edit-delete"), i18n("Re&move Group"),    this );
-  renameGroup_   = new KAction( KIcon("edit"),        i18n("Re&name Group"),    this );
+  renameGroup_   = new KAction( KIcon("edit-rename"), i18n("Re&name Group"),    this );
 
   // Connect the actions
   connect( moveGroupDown_, SIGNAL(activated()),   this,  SLOT(     slotMoveGroupDown()) );
@@ -784,6 +784,8 @@ void KMessView::slotEmailLabelClicked()
 // A group was expanded or collapsed
 void KMessView::slotGroupChanged( const QModelIndex &index )
 {
+  Q_ASSERT( viewModel_ != 0 );
+
   // When searching, the expanded status of the groups is altered, but the settings
   // should not be applied to the actual groups
   if( ! viewModel_->filterRegExp().isEmpty() )
@@ -2002,6 +2004,8 @@ void KMessView::slotUpdateEmailDisplay()
 // Update the expanded status of the list groups
 void KMessView::slotUpdateGroups()
 {
+  Q_ASSERT( viewModel_ != 0 );
+
   // When searching, the expanded status of the groups is altered, but the settings
   // should not be applied to the actual groups
   if( ! viewModel_->filterRegExp().isEmpty() )
