@@ -24,6 +24,8 @@
 #include "model/contactlistmodelitem.h"
 #include "utils/richtextparser.h"
 #include "kmessdebug.h"
+#include "model/contactlist.h"
+#include "contact/contact.h"
 
 #include <QApplication>
 #include <QLabel>
@@ -33,6 +35,7 @@
 
 #include <KIconLoader>
 #include <KLocale>
+#include <KGlobalSettings>
 
 
 /// Default spacing between items
@@ -171,14 +174,7 @@ void KMessViewDelegate::paint( QPainter *painter, const QStyleOptionViewItem &op
     // Only load the picture if it must be shown
     if( displayPictureSize != 0 )
     {
-      picturePixmap.load( itemData[ "displayPicture" ].toString() );
-      if ( ! picturePixmap.isNull() )
-      {
-        // Fast scaled
-        picturePixmap = picturePixmap.scaled( displayPictureSize
-                                            , displayPictureSize
-                                            , Qt::KeepAspectRatio );
-      }
+      picturePixmap = CurrentAccount::instance()->getContactList()->getContactByHandle( itemData[ "handle" ].toString() )->getScaledDisplayPicture();
     }
 
     // Determine which status pixmap to load
@@ -515,8 +511,18 @@ QSize KMessViewDelegate::sizeHint( const QStyleOptionViewItem &option, const QMo
   // Read the actual data off the model index
   const ModelDataList& itemData( index.data().toMap() );
 
-  const QSize size( KIconLoader::SizeSmall + 2, KIconLoader::SizeSmall + 4 );
-
+  // the height of the item should be 
+  // large enough to accommodate the font.
+  QFont generalFont = KGlobalSettings::generalFont();
+  QFontMetricsF metrics( generalFont );
+  
+  QSize size( KIconLoader::SizeSmall + 2, KIconLoader::SizeSmall + 4 );
+  
+  if ( size.height() < metrics.height() )
+  {
+    size.setHeight( metrics.height() );
+  }
+  
   if( itemData.isEmpty() )
   {
     return size;
@@ -539,13 +545,20 @@ QSize KMessViewDelegate::sizeHint( const QStyleOptionViewItem &option, const QMo
           pictureSize.rheight() = picturesDimension + 2;
         }
 
+        qreal fontHeight = metrics.height() * 2; // x2 to account for PSM.
+        
+        if ( fontHeight > pictureSize.height() )
+        {
+          pictureSize.setHeight( fontHeight );
+        }
+        
         return pictureSize;
       }
       else
       {
         return size;
       }
-
+    
     case ContactListModelItem::ItemGroup:
     default:
       return size;
