@@ -35,6 +35,7 @@
  */
 int main(int argc, char *argv[])
 {
+  Q_INIT_RESOURCE( isfqtresources );
   // Init about dialog.
   // Tab 1: General
   KAboutData aboutData( "kmess",                                   // internal name
@@ -48,7 +49,8 @@ int main(int argc, char *argv[])
                               "(c) 2007-2009, Valerio Pilo\n"
                               "(c) 2008-2009, Antonio Nastasi\n"
                               "(c) 2008-2009, Ruben Vandamme\n"
-                              "(c) 2009, Sjors Gielen\n"),
+                              "(c) 2009, Sjors Gielen\n"
+                              "(c) 2009, Adam Goossens\n"),
                         KLocalizedString(),
                         "http://www.kmess.org/",                   // home page
                         "bugs" "@" "kmess" "." "org"               // address for bugs
@@ -173,6 +175,10 @@ int main(int argc, char *argv[])
   // Vincent Fretin (MSN6 emoticon definitions) already above
   aboutData.addCredit( ki18n("Damien Sandras"),            ki18n("GnomeMeeting developer"),          "dsandras" "@" "seconix" "." "com");
   aboutData.addCredit( ki18n("Tobias Tönjes"),             ki18n("Guy with a bag over his head"),    "");
+  aboutData.addCredit( ki18n("Camille Begue"),             ki18n("Chat History functionality when disconnected, autologin checkbox on login screen"),    "prsieux" "@" "@gmail" "." "com");
+  aboutData.addCredit( ki18n("David López"),               ki18n("Nudge button in chat"),            "grannost" "@" "gmail" "." "com" );
+  aboutData.addCredit( ki18n("Pieterjan Camerlynck"),      ki18n("Roaming Service support"),         "pieterjan" "." "camerlynck" "@" "gmail" "." "com" );
+  aboutData.addCredit( ki18n("Anastasios Bourazanis"),     ki18n("Emoticon preview in settings page,clickable contact properties dialog text"), "a.brzns" "@" "gmail" "." "com");
 
   // Other apps
   aboutData.addCredit( ki18n("KMerlin (kmerlin.olsd.de)"), ki18n("Inspiration and assorted code"));
@@ -200,6 +206,9 @@ int main(int argc, char *argv[])
   // make sure we can run tests fast.
 #ifdef KMESSTEST
   options.add( "runtest <test>", ki18n("Run a debug test (developer build only)") );
+  options.add( "server <address>",
+               ki18n("Connect to the specified server instead of the official Live server.\n"
+                     "Use \"localhost\" or \"127.0.0.1\" to connect to a local KMess Test Server.") );
 #endif
 
 
