@@ -610,7 +610,7 @@ void MsnConnection::parseError( const QStringList &command, const QByteArray &pa
       break;
     case 711:
       errorType = MsnSocketBase::ERROR_SERVER;
-      errorMessage = i18n("Write is blocking");
+      errorMessage = i18nc("Error received from the MSN servers", "Unknown error");
       break;
     case 712:
       errorType = MsnSocketBase::ERROR_SERVER;
@@ -653,7 +653,8 @@ void MsnConnection::parseError( const QStringList &command, const QByteArray &pa
   // We've received a warning message: show it in a dialog box, but do not disconnect
   else if( isWarning )
   {
-    KMessageBox::error( 0, errorMessage, i18nc( "Error dialog box title", "MSN Error" ) );
+    KMessageBox::error( 0, errorMessage, i18nc( "Error dialog box title with error number",
+                                                "MSN Error %1", command[0].toInt() ) );
     return;
   }
 
