@@ -42,6 +42,7 @@ class PassportLoginService;
 class QAbstractItemModel;
 class OfflineImService;
 class AddressBookService;
+class RoamingService;
 
 
 
@@ -165,7 +166,7 @@ class MsnNotificationConnection : public MsnConnection
     // Change the personal message of the user.
     void                 changePersonalMessage( QString newMessage );
     // Change the personal properties
-    void                 changePersonalProperties( const QString& friendlyname, const QString& cid, int blp );
+    void                 changePersonalProperties( const QString& cid, int blp );
     // Change a user property
     void                 changeProperty( QString type, QString value );
     // Change a contact property
@@ -190,13 +191,16 @@ class MsnNotificationConnection : public MsnConnection
   private slots:
     // The socket connected, so send the version command.
     void                 slotConnected();
+    // Create the Roaming service
+    RoamingService       *createRoamingService();
     // Show warning notifications
     void                 slotWarning( const QString &warning, bool isImportant );
     // Shows error notifications
     void                 slotError( QString error, MsnSocketBase::ErrorType type );
     // Displays additional info about network errors
     void                 slotErrorEventActivated( NotificationManager::EventSettings settings, NotificationManager::Buttons button );
-
+    // Go online
+    void                 goOnline();
 
   private: // Private methods
     // Create and return one pointer to address book service
@@ -208,8 +212,6 @@ class MsnNotificationConnection : public MsnConnection
     // Create triple des encryption
     QByteArray           createTripleDes ( const QByteArray key, const QByteArray secret,
                                             const QCA::InitializationVector& iv );
-    // Go online
-    void                 goOnline();
     // Received response to adl command
     void                 gotAdl( const QStringList& command, const QString &payload = "" );
     // Received confirmation of the user's status change
@@ -352,6 +354,8 @@ class MsnNotificationConnection : public MsnConnection
     int                  lastStatus_;
     // A timer to watch over the login process
     QTimer               loginTimer_;
+    // A timer used to unlock notifications after the last ILN is received.
+    QTimer               notificationUnlockTimer_;
     // The received offline messages.
     QList<OfflineImMessage*>  offlineImMessages_;
     // The Offline-IM SOAP client
@@ -360,8 +364,12 @@ class MsnNotificationConnection : public MsnConnection
     QList<ChatInformation*>   openRequests_;
     // The object than handles the passport login
     PassportLoginService      *passportLoginService_;
+    // The Roaming SOAP client
+    RoamingService            *roamingService_;
     // The pending offline-im messages
     QStringList               pendingOfflineImMessages_;
+    // Whether the initial status was already sent. See goOnline()
+    bool                      initialStatusSent_;
 
 
   signals:
