@@ -1374,12 +1374,17 @@ void MsnSwitchboardConnection::parseMimeMessage(const QStringList& command, cons
     const MimeMessage& subMessage( message.getBody() );
     parseDatacastMessage( contactHandle, subMessage );
   }
-  else if( contentType == "image/gif" || contentType == "application/x-ms-ink" )
+  else if( contentType == "image/gif" )
   {
-    // This is an ink message.
+    // This is a GIF ink message.
     // Source: http://msdn.microsoft.com/en-us/library/ms818340.aspx
     // warning: WLM doesn't even seem to adhere to that standard...
-    emit gotInkMessage( message.getBody(), contactHandle );
+    emit gotInkMessage( message.getBody(), contactHandle, FORMAT_GIF );
+  }
+  else if( contentType == "application/x-ms-ink" )
+  {
+    // This is an ISF (Ink Serialized Format) ink message.
+    emit gotInkMessage( message.getBody(), contactHandle, FORMAT_ISF );
   }
   else if( contentType == "text/x-clientcaps" )
   {
@@ -1722,43 +1727,87 @@ void MsnSwitchboardConnection::sendChatMessage( const QString& text )
 
 
 // Send a ink to the contact(s)
-void MsnSwitchboardConnection::sendInk( const QByteArray& ink )
+void MsnSwitchboardConnection::sendInk( InkFormat format, const QByteArray& ink )
 {
   bool capable = true;
   ContactBase *contact;
 
+  int inkCapabilityFlag;
+  switch( format )
+  {
+    case FORMAT_ISF:
+      inkCapabilityFlag = ContactBase::MSN_CAP_INK_ISF;
+      break;
+    case FORMAT_GIF:
+      inkCapabilityFlag = ContactBase::MSN_CAP_INK_GIF;
+      break;
+  }
+
+  // Verify whether all contacts are capable to view this Ink message or not
+  ContactBase *unsupportingContact;
   foreach( const QString &handle, contactsInChat_ )
   {
     contact = currentAccount_->getContactByHandle( handle );
-
-    if( ! contact->hasCapability( ContactBase::MSN_CAP_MULTI_PACKET ) ||
-        ! contact->hasCapability( ContactBase::MSN_CAP_INK_GIF)        )
+    if( ! contact->hasCapability( ContactBase::MSN_CAP_MULTI_PACKET )
+    ||  ! contact->hasCapability( (ContactBase::MsnClientCapabilities)inkCapabilityFlag ) )
     {
+      unsupportingContact = contact;
       capable = false;
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+      kDebug() << "Contact" << handle << "does not support our wink. "
+               << "Has multipacket?" << contact->hasCapability( ContactBase::MSN_CAP_MULTI_PACKET )
+               << "Has this ink type support?" << contact->hasCapability( (ContactBase::MsnClientCapabilities)inkCapabilityFlag );
+#endif
       break;
     }
   }
 
   if( ! capable )
   {
-    return;
+    emit showWarning( WARNING_INK_UNSUPPORTED_BY_CONTACT, unsupportingContact );
+
+    // If the only contact in chat doesn't support it, don't send it at all.
+    if( contactsInChat_.count() == 1 )
+    {
+#ifdef KMESSDEBUG_SWITCHBOARD_GENERAL
+      kDebug() << "None of the contacts support the wink, aborting.";
+#endif
+      return;
+    }
   }
 
-  // Create the message for ink
-  // Source: http://msdn.microsoft.com/en-us/library/ms818340.aspx
-  // To send the WLM 2009 content-type (maybe ISF):
-  //message.addField( "Content-Type",    "application/x-ms-ink" );
-  // According to the specs we also need to send:
-  //message.addField( "Content-Transfer-Encoding", "base64" );
-  // but even WLM itself doesn't do that.
+  // Create the Ink message
   MimeMessage message;
-  message.addField("MIME-Version",    "1.0");
-  message.addField("Content-Type",    "image/gif");
-  message.setBody( "base64:" + QString( ink.toBase64() ) );
-  MultiPacketMessage multiMessage( message );
+  message.addField( "MIME-Version", "1.0" );
 
-  // Send the ink packet
-  sendMimeMessageWhenReady( ACK_NAK_ONLY, multiMessage );
+  switch( format )
+  {
+    case FORMAT_ISF:
+      // Source: http://msdn.microsoft.com/en-us/library/ms818340.aspx
+      // According to the specs we'd also need to send:
+      // message.addField( "Content-Transfer-Encoding", "base64" );
+      // but even WLM itself doesn't do that.
+
+      message.addField( "Content-Type", "application/x-ms-ink" );
+      break;
+    case FORMAT_GIF:
+      message.addField( "Content-Type", "image/gif" );
+      break;
+  }
+
+  message.setBody( "base64:" + QString( ink.toBase64() ) );
+
+  // Send the ink packet, if necessary within a Multi Packet message
+  if( message.getBody().size() > 1400 )
+  {
+    MultiPacketMessage multiPartMessage( message );
+
+    sendMimeMessageWhenReady( MsnConnection::ACK_NAK_ONLY, multiPartMessage );
+  }
+  else
+  {
+    sendMimeMessageWhenReady( MsnConnection::ACK_NAK_ONLY, message );
+  }
 }
 
 
