@@ -569,7 +569,7 @@ void AddressBookService::parseAddressBook( const QDomElement &body )
         }
       }
 
-      emit gotPersonalInformation( friendlyName, cid, blp );
+      emit gotPersonalInformation( cid, blp );
       continue;
     }
 
