@@ -19,6 +19,7 @@
 
 #include "../../utils/kmessshared.h"
 #include "../../utils/xmlfunctions.h"
+#include "../../kmessapplication.h"
 #include "../../kmessdebug.h"
 #include "../mimemessage.h"
 #include "soapmessage.h"
@@ -335,9 +336,14 @@ void HttpSoapConnection::sendNextRequest()
   request.setRawHeader( "Connection",                         "Keep-Alive"               );
   request.setRawHeader( "Cache-Control",                      "no-cache"                 );
 
-#if KMESS_ENABLE_LOCAL_MODE == 1
-  // NOTE Local mode: see src/kmessdebug.h
-  request.setUrl( QUrl( "https://127.0.0.1:4430/" ) );
+#ifdef KMESSTEST
+  // Redirect to another server if started with --server
+  if( static_cast<KMessApplication*>( kapp )->getUseTestServer() )
+  {
+    endpoint.setHost( static_cast<KMessApplication*>( kapp )->getTestServer() );
+    endpoint.setPort( 4430 );
+    request.setUrl( endpoint );
+  }
 #endif
 
   if( ! soapAction.isNull() )
