@@ -732,6 +732,17 @@ void PassportLoginService::requestMultipleSecurityTokens()
                 "    <wsse:PolicyReference URI=\"MBI_SSL\"></wsse:PolicyReference>\n"
                 "  </wst:RequestSecurityToken>\n"
 
+                // Authentication for the storage msn site ( for roaming service )
+                "  <wst:RequestSecurityToken Id=\"RST5\">\n"
+                "  <wst:RequestType>http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue</wst:RequestType>\n"
+                "    <wsp:AppliesTo>\n"
+                "      <wsa:EndpointReference>\n"
+                "        <wsa:Address>storage.msn.com</wsa:Address>\n"
+                "      </wsa:EndpointReference>\n"
+                "    </wsp:AppliesTo>\n"
+                "    <wsse:PolicyReference URI=\"MBI\"></wsse:PolicyReference>\n"
+                "  </wst:RequestSecurityToken>\n"
+
                 "</ps:RequestMultipleSecurityTokens>" );
 
   isWaitingForNewTokens_ = true;
